<?php

namespace catchAdmin\logs\model;

use catcher\base\CatchModel as Model;
use catchAdmin\permissions\model\DataRangScopeTrait;

use catchAdmin\system\model\SysDictData;
class WxPushResultLog extends Model
{
    // 表名
    public $name = 'wx_push_result_log';
    use DataRangScopeTrait;
    // 数据库字段映射
    public $field = array(
        'id',
        // 推送人
        'username',
        // 推送结果
        'result',
        // 告警RFID
        'device_number',
        // 创建人ID
        'creator_id',
        // 创建时间
        'created_at',
        // 更新时间
        'updated_at',
        // 软删除
        'deleted_at',
        // 告警id
        'alarm_id',
        // 告警类型
        'alarm_type',
    );

    public function getList()
    {
        return $this->dataRange()
            ->catchSearch()
            ->append(['alarm_type_name'])
            ->order($this->aliasField('id'), 'desc')
            ->paginate();
    }

    public function searchUserNameAttr($query,$value,$data){
        return $query->where('username',$value);
    }
    public function searchDeviceNumberAttr($query,$value,$data){
        return $query->where('device_number',$value);
    }
    public function searchAlarmIdAttr($query,$value,$data){
        return $query->where('alarm_id',$value);
    }

     /**
     * 获取告警类型名称
     */
    public function getResultAttr($value)
    {
       $res=json_decode($value,true);
       if($res['success'] && $res['message']=='ok'){
           return '发送成功';
       }else{
           return $res['message'];
       }
    }
    
    /**
     * 获取告警类型名称
     */
    public function getAlarmTypeNameAttr($value)
    {
        $type = $this->getAttr('alarm_type');
        if (is_null($type)) {
            return '';
        }
        return $this->getDictValue('AlarmReason', $type);
    }

    /*
    * 获取字典值（文本）
    */
   public function getDictValue($typeCode,$value)
   {
       return (new SysDictData())->getValueByCode($typeCode,$value);
   }
}