<?php

namespace catchAdmin\permissions\model;

use catcher\base\CatchModel as Model;
use \think\facade\Cache;

class SysConfig extends Model
{
    // 表名
    public $name = 'sys_config';
    // 数据库字段映射
    public $field = array(
        'id',
        'department_id',
        'device_id',
        'field',
        'fieldValue',
        'type',
        // 创建人ID
        'creator_id',
        // 创建时间
        'created_at',
        // 更新时间
        'updated_at',
        // 软删除
        'deleted_at',
    );

    /**
     * 获取根据类型分组的数据
     */
    public function getListGroupByType()
    {
        $group_data = [];
        $list = $this->catchSearch()
                    ->select()
                    ->toArray();
        foreach ($list as $val) {
            $group_data[$val['type']][$val['field']] = $val['fieldValue'];
        }
        return $group_data;
    }

    /**
     * 获取离线配置值
     */
    public static function getOfflineConfigValue(String $field)
    {
        if (Cache::has($field)) {
            return Cache::get($field);
        }

        return SysConfig::where('type', 'offline_config')
                        ->where('field', $field)
                        ->cache($field, 60)
                        ->value('fieldValue');
    }

    public function getConfigListByType($type)
    {
        $list = $this->catchSearch()
                    ->where('type','in',$type)
                    ->select()
                    ->toArray();
        return $list;
    }
     /**
     * 根据字段和类型获取配置值
     */
    public static function getConfigValueBy(String $field,String $type)
    {
        return SysConfig::where('type', $type)
                        ->where('field', $field)
                        ->cache($type.'_'.$field, 60)
                        ->value('fieldValue');
    }
}