<?php

namespace catchAdmin\stations\model;

use catcher\base\CatchModel as Model;
use catchAdmin\permissions\model\DataRangScopeTrait;
use catchAdmin\stations\model\StationGet;
use catchAdmin\stations\model\StationSearch;

class Station extends Model
{
    use DataRangScopeTrait;
    use StationGet;
    use StationSearch;
    // 表名
    public $name = 'station';
    // 数据库字段映射
    public $field = array(
        'id',
        // 基站名称
        'station_name',
        // 基站Mac
        'station_mac',
        // 基站简码
        'station_code',
        // 基站类型
        'station_type',
        // 经度
        'longitude',
        // 纬度
        'latitude',
        // x轴坐标
        'coordinate_x',
        // y轴坐标
        'coordinate_y',
        // 出入口id
        'access_id',
        // 省
        'province_id',
        // 市
        'city_id',
        // 县区
        'district_id',
        // 在线时间
        'online_time',
        // 安装图片
        'install_photo',
        // 基站朝向 0-未知 1-朝外 2-朝内
        'orientation',
        // 学校id
        'school_id',
        // 基站型号
        'station_model',
        // 基站串号
        'station_sn',
        // 基站版本
        'station_version',
        // WAN口IP
        'wan_ip',
        // 网络模式 0-静态地址，1-动态获取，2-PPPOE拨号，3-3G拨号，4-4G拨号
        'net_model',
        // 信号强度（dbm)
        'rssi',
        // sim卡序列号
        'sim_serial_number',
        // 话机编号
        'cmcc_phone_id',
        // 考勤配置
        'attendance_config',
        // 信号灵敏度 默认62，大于此值会被过滤
        'rssi_sensitive',
        // 节流 0-关闭，1-开启
        'throttling',
        // 安装地址
        'address',
        // 推送企业微信0-否，1-是
        'enable_push_qywx',
        // 建筑id
        'building_id',
        // 楼层id
        'floor_id',
        //房间
        'room_no',
        // 创建人ID
        'creator_id',
        // 创建时间
        'created_at',
        // 更新时间
        'updated_at',
        // 软删除
        'deleted_at',
    );
    /**
     * 获取基站列表
     */
    public function getStationList($field,$order)
    {
        $res =  $this->dataRange()
            ->catchSearch()
            ->append(['school_name', 'building_name', 'floor_name', 'push_text', 'throttling_text', 'orientation_text','command'])
            ->order($this->aliasField($field), $order)
            ->paginate();
        return $res;
    }

    /**
    * 获取所有基站数据(no limit)
    */
    public function getRangeData(){
        $res = $this->dataRange()
                ->catchSearch()
                ->append(['area_id','province_name','city_name','district_name','school_name','access_name','building_name','floor_name','push_text','throttling_text','orientation_text'])
                ->order($this->aliasField('id'), 'desc')
                ->select();
        return $res;

    }
    
    /**
     * 检测基站MAC格式是否正确
     */
    public function checkStationMacFormat($mac)
    {
        $mac = strtoupper(trim($mac));
        $regex = "/^([A-F0-9]{12})$/";

        if (!preg_match($regex, $mac)) {
            return false;
        }
        return true;
    }

    /**
     * 检测设备mac是否重复
     */
    public function isStationMacDuplicate($mac, $id = '')
    {
        $cond = [['station_mac', '=', strtoupper(trim($mac))]];
        if ($id) {
            $cond[] = ['id', '<>', $id];
        }
        return $this->where($cond)->count();
    }
    /**
     * 导出基站列表
     * 
     */
    public function getExportList()
    {
        $res =  $this->dataRange()
            ->catchSearch()
            ->append(['area_id', 'province_name', 'city_name', 'district_name', 'school_name', 'access_name', 'building_name', 'floor_name', 'push_text', 'throttling_text', 'orientation_text'])
            ->order($this->aliasField('id'), 'desc')
            ->select();
        return $res;
    }
}
