<?php

declare(strict_types=1);

namespace tcpclient;

class TcpClient {
             
 
    private $host ;
     
    private $port ;
     
    private $socket ;
     
    private $error ;
     
    private $result = '';
     
 
  
    public  function __construct( $host, $port ){
        $this->host = $host;
        $this->port = $port;
    }
     
  
    public  function send( $msg, $len = 0, $isbin = false ){
        $this->socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if ($this->socket === FALSE) {
            return array('success' => false,
                         'message' => "socket创建失败: " . socket_strerror(socket_last_error()));
        }
        socket_set_option($this->socket, SOL_SOCKET, SO_SNDTIMEO, array('sec' => 3, 'usec' => 0));
        socket_set_option($this->socket, SOL_SOCKET, SO_RCVTIMEO, array('sec' => 10, 'usec' => 0));

        if (!$this->connect()) {
            socket_close($this->socket);
            return array('success' => false, 'message' => $this->error,'code'=>'10000');
        }
        if (!$isbin) {
            if (!$this->write($msg . chr(0), strlen($msg) + 1)) {
                socket_close($this->socket);
                return array('success' => false, 'message' => $this->error,'code'=>'10001');
            }
        } else {
            if (!$this->write($msg, strlen($msg))) {
                socket_close($this->socket);
                return array('success' => false, 'message' => $this->error,'code'=>'10002');
            }
        }

        if (!$this->read()) {
            socket_close($this->socket);
            return array('success' => false, 'message' => $this->error,'code'=>'10003');
        }
        $ret = json_decode(trim($this->result), true);
        if (is_array($ret)){
            return $ret;
        }
    
        return array('success' => false, 'message' => '返回数据格式错误:' .$this->result);
    }
     
  
    private  function connect(  ){
        $timeout = 5;
        socket_set_block($this->socket);
        $time = time();
        while (!@socket_connect($this->socket, $this->host, $this->port)) {
            $err = socket_last_error($this->socket);
            if($err === 56) {
                //echo 'connected ok';
                break;
            }
            if ((time() - $time) >= $timeout) {
                $this->error = '连接超时';
                return false;
            }
            usleep(250000);
        }
        socket_set_block($this->socket);
        return true;
    }
     
  
    private  function write( $msg, $length ){
        while(true) {
            $sent = socket_write($this->socket, $msg, $length);
            if($sent === false) {
                $this->error = "发送失败" . socket_strerror(socket_last_error());;
                return false;
            }
            if($sent < $length) {
                $msg = substr($msg, $sent);
                $length -= $sent;
                //echo "这些字节没法送 :$msg:重新发送";
                $this->error = "发送失败,信息发送不完整";
                return false;
            } else {
                return true;
            }
        }
    }
     
  
    private  function read(  ){
        socket_clear_error();
        while (true) {
            $out = socket_read($this->socket, 2048);
            if ($out == false)
                break;
            $this->result .= $out;

        }
         
        $errorCode = socket_last_error($this->socket);

        if ($errorCode == 115 || $errorCode == 114 || $errorCode == 104) {
                    //  echo "\n连接关闭\n";
            return true;
        } else if ($errorCode == 11) { //读取超时
            $this->error = "读取超时";
            return false;
        } else {
            // $this->error = "读取失败:" . socket_strerror($errorCode);
        }
        return true;
    }
     
 
}