<?php

namespace thirdapi;
use think\facade\Cache;
use think\facade\Db;
use think\Exception;

class ThirdDeviceApi
{

    /**
     * 获取token
     */
    public static function getThirdTokenInfo(){
        return json_decode('{"token":"WSE1o5mbRXCcfgWD","expiredAt":1629710782181,"type":1,"holder":"5E18DC2D7A800000"}', true);
        $tokenRedis = Cache::store('redis')->handler();
        
        
        $tokenInfo = $tokenRedis->hget('third_token_info', 'swager');

        $tokenInfo = json_decode( $tokenInfo, true );
        if($tokenInfo && ( $tokenInfo['expiredAt'] / 1000 - time() ) > 30  ){ //存在并且过期时间距现在大于30秒
            return $tokenInfo;
        }
        
        
        
        $url = "http://rinlink-a18.beijing-cn-k8s-test.rinlink.com/token/user/app";
        $headers[]  =  "Content-Type: application/json";
        
        $postData = [
            'username' => 'swager',
            'password' => '123456'
        ];
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt( $ch, CURLOPT_HTTPHEADER, $headers );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt( $ch, CURLOPT_POSTFIELDS,json_encode($postData) );
        $output = curl_exec($ch);
        curl_close($ch);
        if($output === false){
            self::push_log('thirdapi_token_curl_error',curl_error($ch));
            return false;
        }
        
        
        
        $tokenRes = json_decode($output, true);
        if($tokenRes['error'] || !$tokenRes['token']){
            self::pushCommonLog('thirdapi_gettoken_response_error',$tokenRes);
            return false;
        }
        $tokenRedis->hset('third_token_info', 'swager', $output);
        
        return $tokenRes;
    }

    /**
     * 注册设备到第三方
     */
    public static function registerDevice($deviceImei){

        $tokenInfo = self::getThirdTokenInfo();
        if(!$tokenInfo){
            return false;
        }
        
        $postData = [
            "deviceImei" => $deviceImei,
            "phone" => "18888888888",
            "userId" => $tokenInfo['holder']
        ];
        
        
        
        $url = "http://rinlink-a18.beijing-cn-k8s-test.rinlink.com/device";
        $headers[]  =  "Content-Type: application/json";
        $headers[]  =  "accept: */*";
        $headers[]  =  "Authorization: ". $tokenInfo['token'];
        
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt( $ch, CURLOPT_HTTPHEADER, $headers );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode($postData) );
        $output = curl_exec($ch);
        
        if (curl_errno($ch)) {
            self::pushCommonLog('thirdapi_devicelocation_curl_error',curl_error($ch));
            curl_close($ch);
            return false;
        }
        curl_close($ch);
        
        $deviceRes = json_decode($output, true) ;
        self::pushCommonLog('thirdapi_register_device_response_error',$deviceRes);

        if(array_key_exists("error",$deviceRes) || array_key_exists("code",$deviceRes)){
            $deviceRes['request_imei'] = $deviceImei;
            self::pushCommonLog('thirdapi_register_device_response_error',$deviceRes);
            return false;
        }
        
        return true;

    }
    /**
     * 获取但三方设备列表
     */
    public static function getDeviceList(){
        $tokenInfo = self::getThirdTokenInfo();
        if(!$tokenInfo){
            return false;
        }

        $headers[]  =  "Content-Type: application/json";
        $headers[]  =  "Authorization: ". $tokenInfo['token'];
        $url = 'http://rinlink-a18.beijing-cn-k8s-test.rinlink.com/device?userId=' . $tokenInfo['holder'];
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_TIMEOUT,5);
        curl_setopt( $ch, CURLOPT_HTTPHEADER, $headers );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $response = curl_exec($ch);
        
        if (curl_errno($ch)) {
            self::pushCommonLog('thirdapi_devicelist_curl_error',curl_error($ch));
            curl_close($ch);
            return false;
        }
        curl_close($ch);
        
        return json_decode($response, true);
    }
    /**
     * 获取但三方设备轨迹
     */
    public static function getHistoryLocations($imei,$option = null){
        $tokenInfo = self::getThirdTokenInfo();
        if(!$tokenInfo){
            return false;
        }


        $pageNo = $option['pageNo'] ? :  0;
        $pageSize = $option['pageSize'] ? : 6000;
        $startTime = $option['startTime'] ? :strtotime(date('Y-m-d 00:00:00') ) * 1000;
        $endTime = $option['endTime'] ? : strtotime( date('Y-m-d 23:59:59')) * 1000;
        $positionTypes = $option['positionTypes'] ? :'1,2';


        $headers[]  =  "Content-Type: application/json";
        $headers[]  =  "Authorization: ". $tokenInfo['token'];
        $url = 'http://rinlink-a18.beijing-cn-k8s-test.rinlink.com/devicedata/location/filter?imei='.$imei.'&pageNo='.$pageNo.'&pageSize='.$pageSize.'&startTime='.$startTime.'&endTime='.$endTime.'&positionTypes='.$positionTypes;
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_TIMEOUT,5);
        curl_setopt( $ch, CURLOPT_HTTPHEADER, $headers );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $response = curl_exec($ch);
        
        if (curl_errno($ch)) {
            self::pushCommonLog('thirdapi_devicelocation_curl_error',curl_error($ch));
            curl_close($ch);
            return false;
        }
        curl_close($ch);
        
        return json_decode($response, true);
    }
    /**
     * 删除第三方设备
     */
    public static function deleteDevice($imei){

        $tokenInfo = self::getThirdTokenInfo();
        if(!$tokenInfo){
            return false;
        }

        $headers[]  =  "Content-Type: application/json";
        $headers[]  =  "Authorization: ". $tokenInfo['token'];
        $url = 'http://rinlink-a18.beijing-cn-k8s-test.rinlink.com/device/'.$imei;
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
    
        //设置头
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers); //设置请求头
    
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);//SSL认证。
        $output = curl_exec($ch);

        if (curl_errno($ch)) {
            self::pushCommonLog('thirdapi_device_delete_curl_error',curl_error($ch));
            curl_close($ch);
            return false;
        }
        curl_close($ch);

        $response = json_decode($output, true);

        if(array_key_exists("error",$response) || array_key_exists("code",$response)){
            $response['request_imei'] = $imei;
            self::pushCommonLog('thirdapi_register_device_response_error',$response);
            return false;
        }
    
        
        return json_decode($output, true);
    }






        /**
     * 写入日志
     */
    public static function pushCommonLog( $filename = 'default', $msg)
    {
        if(is_array($msg)){
            $msg = json_encode($msg);
        }
        $file = runtime_path() . '/log/' . date("Ymd", time()) . "/".$filename."log";
        $folder = dirname($file);
        if (!is_dir($folder)) {
            mkdir($folder, 0777, true);
        }
        file_put_contents($file, '[' . date('Y-m-d H:i:s') . ']' . $msg . PHP_EOL, FILE_APPEND);
    }
 
}
